"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.update = exports.validateFromUrl = exports.validate = void 0;
const util_1 = __importDefault(require("util"));
const path_1 = __importDefault(require("path"));
const child_process_1 = require("child_process");
const fs_extra_1 = __importDefault(require("fs-extra"));
const readline_sync_1 = __importDefault(require("readline-sync"));
const utils_1 = require("./utils");
const temp_1 = __importDefault(require("temp"));
const crypto_1 = __importDefault(require("crypto"));
crypto_1.default.constants.SSL_OP_ALLOW_UNSAFE_LEGACY_RENEGOTIATION;
function validate(dataFile, schemaVersion, options) {
    return __awaiter(this, void 0, void 0, function* () {
        // check to see if supplied json file exists
        if (!fs_extra_1.default.existsSync(dataFile)) {
            console.log(`Could not find data file: ${dataFile}`);
            process.exitCode = 1;
            return;
        }
        // get the schema that matches the chosen version and target name. then, use it to validate.
        (0, utils_1.useRepoVersion)(schemaVersion, options.target, options.strict).then(schemaPath => {
            if (schemaPath != null) {
                (0, utils_1.runContainer)(schemaPath, dataFile, options.out);
            }
            else {
                console.log('No schema available - not validating.');
                process.exitCode = 1;
            }
        });
    });
}
exports.validate = validate;
function validateFromUrl(dataUrl, schemaVersion, options) {
    return __awaiter(this, void 0, void 0, function* () {
        temp_1.default.track();
        try {
            if (yield (0, utils_1.checkDataUrl)(dataUrl)) {
                return (0, utils_1.useRepoVersion)(schemaVersion, options.target, options.strict).then((schemaPath) => __awaiter(this, void 0, void 0, function* () {
                    if (schemaPath != null) {
                        const dataFile = yield (0, utils_1.downloadDataFile)(dataUrl, temp_1.default.mkdirSync());
                        if (typeof dataFile === 'string') {
                            return yield (0, utils_1.runContainer)(schemaPath, dataFile, options.out);
                        }
                        else {
                            let continuation = true;
                            // we have multiple files, so let's choose as many as we want
                            while (continuation === true) {
                                const chosenEntry = (0, utils_1.chooseJsonFile)(dataFile.jsonEntries);
                                yield (0, utils_1.getEntryFromZip)(dataFile.zipFile, chosenEntry, dataFile.dataPath);
                                yield (0, utils_1.runContainer)(schemaPath, dataFile.dataPath, options.out);
                                continuation = readline_sync_1.default.keyInYNStrict('Would you like to validate another file in the ZIP?');
                            }
                            dataFile.zipFile.close();
                        }
                    }
                    else {
                        console.log('No schema available - not validating.');
                        process.exitCode = 1;
                    }
                }));
            }
            else {
                console.log('Exiting.');
                process.exitCode = 1;
            }
        }
        finally {
            temp_1.default.cleanupSync();
        }
    });
}
exports.validateFromUrl = validateFromUrl;
function update() {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            // check if the repo exists. if not, clone it. if it exists, fetch updates.
            if (!fs_extra_1.default.existsSync(path_1.default.join(utils_1.config.SCHEMA_REPO_FOLDER, '.git'))) {
                yield util_1.default.promisify(child_process_1.exec)(`git clone ${utils_1.config.SCHEMA_REPO_URL} "${utils_1.config.SCHEMA_REPO_FOLDER}"`);
                console.log('Retrieved schemas.');
            }
            else {
                yield util_1.default.promisify(child_process_1.exec)(`git -C "${utils_1.config.SCHEMA_REPO_FOLDER}" checkout master && git -C "${utils_1.config.SCHEMA_REPO_FOLDER}" pull --no-rebase -t`);
                console.log('Updated schemas.');
            }
        }
        catch (error) {
            console.log(`Error when updating available schemas: ${error}`);
            process.exitCode = 1;
        }
    });
}
exports.update = update;
