#!/usr/bin/env node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const commander_1 = require("commander");
const commands_1 = require("./commands");
const utils_1 = require("./utils");
main().catch(error => {
    console.log(`Encountered an unexpected error: ${error}`);
});
function main() {
    return __awaiter(this, void 0, void 0, function* () {
        commander_1.program
            .name('cms-mrf-validator')
            .description('Tool for validating health coverage machine-readable files.')
            .command('validate')
            .description('Validate a file against a specific published version of a CMS schema.')
            .usage('<data-file> <schema-version> [options]')
            .argument('<data-file>', 'path to data file to validate')
            .argument('<schema-version>', 'version of schema to use for validation')
            .option('-o, --out <out>', 'output path')
            .addOption(new commander_1.Option('-t, --target <schema>', 'name of schema to use')
            .choices(utils_1.config.AVAILABLE_SCHEMAS)
            .default('in-network-rates'))
            .option('-s, --strict', 'enable strict checking, which prohibits additional properties in data file')
            .action(commands_1.validate);
        commander_1.program
            .command('from-url')
            .description('Validate the file retrieved from a URL against a specific published version of a CMS schema.')
            .usage('<data-url> <schema-version> [options]')
            .argument('<data-url>', 'URL to data file to validate')
            .argument('<schema-version>', 'version of schema to use for validation')
            .option('-o, --out <out>', 'output path')
            .addOption(new commander_1.Option('-t, --target <schema>', 'name of schema to use')
            .choices(utils_1.config.AVAILABLE_SCHEMAS)
            .default('in-network-rates'))
            .option('-s, --strict', 'enable strict checking, which prohibits additional properties in data file')
            .action(commands_1.validateFromUrl);
        commander_1.program
            .command('update')
            .description('Update the available schemas from the CMS repository.')
            .action(commands_1.update);
        commander_1.program.parseAsync(process.argv);
    });
}
